import pandas as pd
import pathlib
import datetime

from wiliot_tools.utils.wiliot_gui.wiliot_gui import WiliotGui

wg = WiliotGui(params_dict={'folder_path': {'text': 'please select folder to merge', 'value': '', 'widget_type': 'file_input', 'options': 'folder'}},
               title='Mrged Data')

values = wg.run()

folder_base = pathlib.Path(values['folder_path'])
file_list_path = list(folder_base.rglob("*.csv"))

df = pd.DataFrame()
for f in file_list_path:
    if 'merged_data_' in f.name:
        print(f'ignore file: {f} continue to the next file')
        continue
    new_df = pd.read_csv(f)
    new_df.insert(loc=len(new_df.columns), column='fileName', value=f.name.replace('.csv', ''))
    new_df.insert(loc=len(new_df.columns), column='folderName', value=f.parent.name)

    df = pd.concat([df, new_df], ignore_index=True)

df.to_csv(folder_base.joinpath(f'merged_data_{datetime.datetime.now().strftime("%d%m%Y_%H%M%S")}.csv'))
print('done')


import threading
from shipment_approval_utils import *


class RequestDetails(ctk.CTkFrame):

    def __init__(self, parent, show_page_callback, request_id=None):
        super().__init__(parent)
        self.tags_tree = None
        self.tags_df = None
        self.parent = parent
        self.show_page_callback = lambda: show_page_callback(self)
        self.tree = None
        self.request_id = request_id

        label = ctk.CTkLabel(self, text="Request Details", font=("Arial", 28, "bold"))
        label.pack(pady=50)
        show_img(self)
        self.tags_cols = ['external_id', 'reason']
        self.show_tags_df(pd.DataFrame(columns=self.tags_cols))
        btn = ctk.CTkButton(self, text="Back", command=self.show_page_callback, width=120)
        btn.place(x=50, y=self.parent.H - 50)

        btn = ctk.CTkButton(master=self, text="Open in Excel",
                            command=lambda: open_csv_with_excel("data/tags_tmp.csv"), width=220,
                            font=('Helvetica', 14))
        btn.place(x=self.parent.W / 2 - 110, y=self.parent.H - 100)

        threading.Thread(target=self.get_tags).start()

    def show_tags_df(self, df):
        self.tags_tree = self.parent.create_tree_view(self, df, equal_width=True, width=100, height=25)
        self.tags_tree.place(x=50, y=150)

    def get_tags(self):
        self.tags_df = self.parent.get_request_tags_info(self.request_id)
        self.tags_tree.destroy()
        self.tags_df['reason'] = self.tags_df['reason'].apply(lambda x: '-' if str(x).lower() == 'nan' else x)
        self.tags_df['external_id'] = self.tags_df['external_id'].apply(lambda x: '-' if str(x).lower() == 'nan' else x)
        self.show_tags_df(self.tags_df)
        self.update_colors_tags()

    def update_colors_tags(self):
        try:
            self.tags_tree.tag_configure('red', foreground='#d44e2a')
        except:
            pass
        for i, item in enumerate(self.tags_tree.get_children()):
            self.tags_tree.item(item, tags=('red',))



"""
Copyright (c) 2016- 2025, Wiliot Ltd. All rights reserved.

Redistribution and use of the Software in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

  2. Redistributions in binary form, except as used in conjunction with
  Wiliot's Pixel in a product or a Software update for such product, must reproduce
  the above copyright notice, this list of conditions and the following disclaimer in
  the documentation and/or other materials provided with the distribution.

  3. Neither the name nor logo of Wiliot, nor the names of the Software's contributors,
  may be used to endorse or promote products or services derived from this Software,
  without specific prior written permission.

  4. This Software, with or without modification, must only be used in conjunction
  with Wiliot's Pixel or with Wiliot's cloud service.

  5. If any Software is provided in binary form under this license, you must not
  do any of the following:
  (a) modify, adapt, translate, or create a derivative work of the Software; or
  (b) reverse engineer, decompile, disassemble, decrypt, or otherwise attempt to
  discover the source code or non-literal aspects (such as the underlying structure,
  sequence, organization, ideas, or algorithms) of the Software.

  6. If you create a derivative work and/or improvement of any Software, you hereby
  irrevocably grant each of Wiliot and its corporate affiliates a worldwide, non-exclusive,
  royalty-free, fully paid-up, perpetual, irrevocable, assignable, sublicensable
  right and license to reproduce, use, make, have made, import, distribute, sell,
  offer for sale, create derivative works of, modify, translate, publicly perform
  and display, and otherwise commercially exploit such derivative works and improvements
  (as applicable) in conjunction with Wiliot's products and services.

  7. You represent and warrant that you are not a resident of (and will not use the
  Software in) a country that the U.S. government has embargoed for use of the Software,
  nor are you named on the U.S. Treasury Department’s list of Specially Designated
  Nationals or any other applicable trade sanctioning regulations of any jurisdiction.
  You must not transfer, export, re-export, import, re-import or divert the Software
  in violation of any export or re-export control laws and regulations (such as the
  United States' ITAR, EAR, and OFAC regulations), as well as any applicable import
  and use restrictions, all as then in effect

THIS SOFTWARE IS PROVIDED BY WILIOT "AS IS" AND "AS AVAILABLE", AND ANY EXPRESS
OR IMPLIED WARRANTIES OR CONDITIONS, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED
WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, NONINFRINGEMENT,
QUIET POSSESSION, FITNESS FOR A PARTICULAR PURPOSE, AND TITLE, ARE DISCLAIMED.
IN NO EVENT SHALL WILIOT, ANY OF ITS CORPORATE AFFILIATES OR LICENSORS, AND/OR
ANY CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES, FOR THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES,
FOR ANY LOSS OF USE OR DATA OR BUSINESS INTERRUPTION, AND/OR FOR ANY ECONOMIC LOSS
(SUCH AS LOST PROFITS, REVENUE, ANTICIPATED SAVINGS). THE FOREGOING SHALL APPLY:
(A) HOWEVER CAUSED AND REGARDLESS OF THE THEORY OR BASIS LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE);
(B) EVEN IF ANYONE IS ADVISED OF THE POSSIBILITY OF ANY DAMAGES, LOSSES, OR COSTS; AND
(C) EVEN IF ANY REMEDY FAILS OF ITS ESSENTIAL PURPOSE.
"""


import os.path
import os
import pandas as pd
import csv
import datetime
import re
import pathlib
from wiliot_core import InlayTypes,PacketList
try:
    from wiliot_tools.utils.wiliot_gui.wiliot_gui import WiliotGui
except Exception as e:
    print(f'could not import wiliot_gui due to: {e}')
DECRYPTION_MODE = False

try:
    from wiliot_core import DecryptedPacket, DecryptedPacketList
    DECRYPTION_MODE = True
    print('Working on decrypted mode')
except Exception as e:
    pass


alpha_packet_fields = ['common_run_name', 'external_id', 'selected_tag', 'fail_bin_str',
                       'test_start_time', 'trigger_time', 'test_end_time']
MAX_PATH_CHARS = 260


def csv_to_dict(path=None):
    if path:
        with open(path, 'r') as f:
            reader = csv.DictReader(f, delimiter=',')
            col_names = reader.fieldnames
            data_out = {col: [] for col in col_names}
            try:
                for row in reader:
                    for col in col_names:
                        data_out[col].append(row[col])
            except Exception as e:
                print("couldn't load csv with the following exception: {}".format(e))
        return data_out
    else:
        print('please provide a path')
        return None

def export_decrypted_data(packet_path_in, is_big_file=False, relevant_columns=None, inlay_type=None, ignore_crc=True, packet_version=None, do_decryption=True, do_resolve=False, api_key='', owner_id=''):
    """

    :param packet_path_in: can be a packet file path or a folder path contains many run folders
                           with packet data at each sub folder
    :type packet_path_in: str
    :param is_big_file: if big data, relevant columns should be specified to shorten the run time
    :type is_big_file: bool
    :param relevant_columns: if big_data is True only the specified columns will be exported
                             according to the decoded data fields
    :type relevant_columns: list or None
    :return: save the decrypted output file with the suffix of _out_<datetime> and
             return a dict where key is the file path and value is dict contains packets_df and statistics_df
    :rtype: dict
    """
    if isinstance(packet_path_in, list):
        file_list = packet_path_in
    elif os.path.isfile(packet_path_in):
        file_list = [packet_path_in]
    elif os.path.isdir(packet_path_in):
        folder_base = pathlib.Path(packet_path_in)
        file_list_path = list(folder_base.rglob("*@packets_data.csv"))
        file_list = [file_in.__str__() for file_in in file_list_path]
        f_list_p2 = list(folder_base.rglob("listener_log_*.log"))
        file_list += [file_in.__str__() for file_in in f_list_p2]
    else:
        raise Exception('packet_path_in is not a file path nor folder : {}'.format(packet_path_in))
    packet_path_out = ''
    de_packet_list = None
    output = {}
    packet_version = float(packet_version) if packet_version is not None else None
    for packet_in in file_list:
        print('process file {}'.format(packet_in))
        timestamp_str = datetime.datetime.now().strftime("%d%m%Y_%H%M%S")
        if packet_in.endswith('.log'):
            print('analyzing packets...')
            packet_list = PacketList()
            packet_list = packet_list.log_file_to_packet_struct(log_path=packet_in, ignore_crc=ignore_crc)
            if do_decryption:
                de_packet_list = DecryptedPacketList.from_packet_list(packet_list=packet_list, packet_version_by_user=packet_version)
            else:
                de_packet_list = packet_list
            packet_path_out = '{}_out_{}.csv'.format(packet_in.split('.log')[0], timestamp_str)
            stat_path_out = '{}_stat_{}.csv'.format(packet_in.split('.log')[0], timestamp_str)
            resolve_path_out = '{}_resolve_{}.csv'.format(packet_in.split('.log')[0], timestamp_str)
        elif packet_in.endswith('.csv') or packet_in.endswith('.json'):
            split_str = packet_in.split('.')[-1]
            packet_path_out = '{}_out_{}.csv'.format(packet_in.split(f'.{split_str}')[0], timestamp_str)
            stat_path_out = '{}_stat_{}.csv'.format(packet_in.split(f'.{split_str}')[0], timestamp_str)
            resolve_path_out = '{}_resolve_{}.csv'.format(packet_in.split(f'.{split_str}')[0], timestamp_str)
            print('analyzing packets...')
            if is_big_file:
                if DECRYPTION_MODE is False:
                    raise Exception('decryption mode is not available, can not run with is_big_file=True')
                else:
                    packet_dict = csv_to_dict(path=packet_in)
                    packet_df = pd.DataFrame(data=packet_dict)
                    decoded_dict = {k: [] for k in relevant_columns}
                    for _, r in packet_df.iterrows():
                        if r['raw_packet'] == '':
                            for k in decoded_dict.keys():
                                decoded_dict[k].append(None)
                        else:
                            p = DecryptedPacket(raw_packet=r['raw_packet'], time_from_start=r['time_from_start'])
                            for k in decoded_dict.keys():
                                decoded_dict[k].append(p.decoded_data[k])

                    for k in decoded_dict.keys():
                        packet_df[k] = decoded_dict[k]
                    packet_df.to_csv(packet_path_out)

            else:
                used_class = DecryptedPacketList if do_decryption else PacketList

                inlay = InlayTypes(inlay_type) if inlay_type else None
                packet_obj = used_class().import_packet_df(
                    path=packet_in,
                    import_all=True,
                    inlay_type=inlay,
                    ignore_crc=ignore_crc,
                )

                de_packet_list = packet_obj
        else:
            print(f'unsupported log file (csv or log) bug got {packet_in}')
            continue

        if not is_big_file:
            de_tag_collection = de_packet_list.to_tag_list()
            de_tag_collection.to_csv(path=get_valid_path_length(packet_path_out),
                                     is_overwrite=True, add_sprinkler_info=True)
            print('save packets data to {}'.format(packet_path_out))

            print('doing tags statistics...')
            stat_df = de_tag_collection.get_statistics()
            stat_df.to_csv(get_valid_path_length(stat_path_out))
            if do_resolve:
                print('doing resolve...')
                resolve_df = pd.DataFrame(columns=["tag", "payload", "external_id"])
                from wiliot_api import ManufacturingClient
                client = ManufacturingClient(api_key=api_key)
                for tag, p_list in de_tag_collection.tags.items():
                    payload = list(p_list.payload_map_list.keys())[0]
                    res = client.safe_resolve_payload(payload=payload, owner_id=owner_id)
                    tag_id = res.get('externalId', 'unknown external id')
                    resolve_df = pd.concat([resolve_df, pd.DataFrame([{'tag': tag, 'payload': payload, 'external_id': tag_id}])], ignore_index=True)
                resolve_df.to_csv(get_valid_path_length(resolve_path_out))
                print('save resolve data to {}'.format(resolve_path_out))

            output[packet_path_out] = {'packets_df': de_tag_collection.get_df(), 'statistics_df': stat_df}

    print('done')
    return output


def get_valid_path_length(path_in):
    if len(path_in) >= MAX_PATH_CHARS:
        file_name = os.path.basename(path_in)
        file_name = file_name[(len(path_in) - MAX_PATH_CHARS + 1):]
        path_in = os.path.join(os.path.dirname(path_in), file_name)
    return path_in


def get_files_path():
    inlay_group = tuple(['']+[x.value for x in InlayTypes])

    value_dict = {
        'packets_data_file': {'text': 'Choose packets_data file or listener log file or \nfolder contains packet_data '
                                      'that you want to analyze:',
                              'value': '',
                              'widget_type': "file_input",
                              'options': 'multiple'},
        'inlay': {'text': 'Select inlay',
                  'value': inlay_group},
        'packet_version': {'text': 'Packet Version (relevant for bridge data only)', 'value': ''},
        'bad_crc_en': {'text': 'Filter out bad crc', 'value': False},
    }
    if DECRYPTION_MODE:
        value_dict['pixie_analyzer_en'] = {'text': 'Run PixieAnalyzer', 'value': False}
        value_dict['do_decryption'] = {'text': 'Do Decryption', 'value': True}
        value_dict['do_resolve'] = {'text': 'Do Resolve', 'value': False}
        value_dict['resolve_owner_id'] = {'text': 'Resolve Owner ID', 'value': 'wiliot-ops'}
        value_dict['resolve_env'] = {'text': 'Resolve Env', 'value': 'prod'}

    gui = WiliotGui(params_dict=value_dict, title='Analyze packet data')
    gui.run()

    return gui.get_all_values()


if __name__ == '__main__':
    try:
        from wiliot_tools.extended.pixie_analyzer.pixie_analyzer import PixieAnalyzer
    except Exception as e:
        if DECRYPTION_MODE:
            print(e)
        pass
    user_input = get_files_path()
    if user_input is None:
        print('no file was selected. closing the app')
    else:
        p_version = user_input['packet_version'] if user_input['packet_version'] != '' else None
        api_key = ''
        owner_id = ''
        if user_input.get('do_resolve', False):
            owner_id = user_input['resolve_owner_id']
            env = user_input['resolve_env']
            if not owner_id or owner_id == '':
                raise Exception('owner id must be provided to do resolve')
            from wiliot_core.utils.utils import GetApiKey
            g = GetApiKey(gui_type='ttk', env=env, owner_id=owner_id, client_type='asset')
            api_key = g.get_api_key()
        export_decrypted_data(
                packet_path_in=user_input['packets_data_file'],
                is_big_file=False,
                relevant_columns=None,
                inlay_type=user_input['inlay'],
                ignore_crc=not user_input['bad_crc_en'],
                packet_version=p_version,
                do_decryption=user_input.get('do_decryption', False),
                do_resolve=user_input.get('do_resolve', False),
                api_key=api_key,
                owner_id=owner_id
            )
            
        if user_input.get('pixie_analyzer_en', False):
            my_analyzer = PixieAnalyzer()
            [text_log, multi_tag, user_event] = my_analyzer.PacketDecoder().parse(
                input=user_input['packets_data_file'])
            a = my_analyzer.plot_graphs(plot_data=multi_tag, user_event=user_event)



import threading
from owner_change_utils import *


class RequestDetails(ctk.CTkFrame):

    def __init__(self, parent, show_page_callback, request_id=None):
        super().__init__(parent)
        self.tags_df = None
        self.parent = parent
        self.show_page_callback = lambda: show_page_callback(self)
        self.tree = None
        self.request_id = request_id

        label = ctk.CTkLabel(self, text="Request Details", font=("Arial", 28, "bold"))
        label.pack(pady=50)
        show_img(self)
        self.tags_cols = ['tagId', 'status', 'comment']
        ctk.CTkLabel(self, text="Reels Details", font=("Arial", 25)).place(x=250, y=100)
        ctk.CTkLabel(self, text="Tags Details", font=("Arial", 25)).place(x=750, y=100)
        reels_df = pd.read_csv(f"data/{request_id}.csv")
        if "To Owner" in reels_df:
            reels_df = reels_df.drop("To Owner", axis=1)
        for col in ['Total Tags', 'Passed', 'Failed']:
            reels_df[col] = '-'
        self.tree = self.parent.create_tree_view(self, reels_df, equal_width=False, width=700, height=25)
        self.tree.place(x=50, y=150)
        self.tree.bind("<<TreeviewSelect>>", self.update_tags_df)

        self.tags_tree = self.parent.create_tree_view(self, pd.DataFrame(columns=self.tags_cols), equal_width=False,
                                                      width=600, height=25)
        self.tags_tree.place(x=550, y=150)
        self.selection_box = ctk.CTkComboBox(master=self,
                                             values=['All', 'Succeeded Only', 'Failed Only'], justify='center',
                                             command=self.update_tags_df)
        self.selection_box.place(x=920, y=100)
        self.selection_box.set('Failed Only')
        btn = ctk.CTkButton(self, text="Back", command=self.show_page_callback, width=120)
        btn.place(x=50, y=self.parent.H - 50)

        btn = ctk.CTkButton(master=self, text="Open in Excel",
                            command=lambda: open_csv_with_excel("data/tags_tmp.csv"), width=220,
                            font=('Helvetica', 14))
        btn.place(x=self.parent.W / 2 - 110, y=self.parent.H - 100)

        threading.Thread(target=self.get_tags).start()

    def update_tags_df(self, _):
        selected_item = self.tree.selection()
        if not selected_item:
            return
        values = self.tree.item(selected_item[0], 'values')
        ext_id_prefix = values[0]
        first_tag = str(values[1]).zfill(4)
        last_tag = str(values[2]).zfill(4)
        if ext_id_prefix != 'All Reels':
            filtered_df = self.tags_df.loc[ext_id_prefix]
            if isinstance(filtered_df, pd.Series):
                filtered_df = pd.DataFrame([filtered_df], columns=filtered_df.index)
            filtered_df = filtered_df[
                (filtered_df['tagId'].str[-4:] >= first_tag) &
                (filtered_df['tagId'].str[-4:] <= last_tag)
                ]
        else:
            filtered_df = self.tags_df
        if self.selection_box.get().lower() != 'all':
            filtered_df = filtered_df[filtered_df['status'] == self.selection_box.get().lower().split(" ")[0]]
        self.tags_tree.destroy()
        self.tags_tree = self.parent.create_tree_view(self, pd.DataFrame(filtered_df[self.tags_cols]),
                                                      equal_width=False, width=600, height=25)
        self.tags_tree.place(x=550, y=150)
        self.update_colors_tags()
        self.update_colors_reels()

    def get_tags(self):
        self.tags_df = self.parent.get_request_tags_info(self.request_id)
        fails_path = os.path.join("data", self.request_id + "_tags.csv")
        if os.path.exists(fails_path):
            fails_df = pd.read_csv(fails_path)
            self.tags_df = pd.merge(left=self.tags_df, right=fails_df, left_on='tagId', right_on='external_id', how='left')
            self.tags_df['comment'] = self.tags_df['fail_bin_str']
        self.tags_tree.destroy()
        self.tags_df['comment'] = self.tags_df['comment'].apply(lambda x: '-' if str(x).lower() == 'nan' else x)
        self.tags_tree = self.parent.create_tree_view(self, pd.DataFrame(columns=self.tags_cols), equal_width=False,
                                                      width=600, height=25)
        self.tags_tree.place(x=550, y=150)
        self.tags_df['prefix'] = self.tags_df['tagId'].str[:-5]
        self.tags_df = self.tags_df.set_index('prefix')
        for item in self.tree.get_children():
            values = self.tree.item(item, "values")
            ext_id_prefix = values[0]
            first_tag = str(values[1]).zfill(4)
            last_tag = str(values[2]).zfill(4)
            filtered_df = self.tags_df.loc[ext_id_prefix]
            if isinstance(filtered_df, pd.Series):
                filtered_df = pd.DataFrame([filtered_df], columns=filtered_df.index)
            total_tags_df = filtered_df[
                (filtered_df['tagId'].str[-4:] >= first_tag) &
                (filtered_df['tagId'].str[-4:] <= last_tag)
                ]
            passed_tags_df = total_tags_df[(total_tags_df['status'] == 'succeeded')]
            failed_tags_df = total_tags_df[(total_tags_df['status'] == 'failed')]
            new_values = list(values)[:3] + [len(total_tags_df), len(passed_tags_df), len(failed_tags_df)]
            self.tree.item(item, values=new_values)
        passed_tags_df = self.tags_df[(self.tags_df['status'] == 'succeeded')]
        failed_tags_df = self.tags_df[(self.tags_df['status'] == 'failed')]
        item_id = self.tree.insert("", 0, values=["All Reels", '-', '-', len(passed_tags_df) + len(failed_tags_df),
                                                  len(passed_tags_df), len(failed_tags_df)])
        self.tree.selection_set(item_id)
        self.update_colors_tags()
        self.update_colors_reels()

    def update_colors_tags(self):
        try:
            self.tags_tree.tag_configure('red', foreground='#d44e2a')
            self.tags_tree.tag_configure('green', foreground='#39a987')
            self.tags_tree.tag_configure('yellow', foreground='#FFBF00')
            self.tags_tree.tag_configure('orange', foreground='orange')
        except:
            pass
        for i, item in enumerate(self.tags_tree.get_children()):
            values = self.tags_tree.item(item, "values")
            if values[1] == 'failed':
                self.tags_tree.item(item, tags=('red',))
            elif values[1] == 'succeeded':
                self.tags_tree.item(item, tags=('green',))

    def update_colors_reels(self):
        try:
            self.tree.tag_configure('red', foreground='#d44e2a')
            self.tree.tag_configure('green', foreground='#39a987')
            self.tree.tag_configure('yellow', foreground='#FFBF00')
            self.tree.tag_configure('orange', foreground='orange')
        except:
            pass
        for i, item in enumerate(self.tree.get_children()):
            values = self.tree.item(item, "values")
            if values[3] != '-' and int(values[4]) == int(values[3]):
                self.tree.item(item, tags=('green',))
            elif values[3] != '-' and int(values[5]) == int(values[3]):
                self.tree.item(item, tags=('red',))
            elif values[3] != '-' and int(values[5]) > 0:
                self.tree.item(item, tags=('orange',))



from pathlib import Path
import json

from wiliot_tools.test_equipment.test_equipment import ZebraPrinter

label = """
^XA

^FX Third section with bar code.
^BY5,2,270
^FO100,550^BC^FD12345078^FS

^FX Fourth section (the two boxes on the bottom).
^FO50,900^GB700,250,3^FS
^FO400,900^GB3,250,3^FS
^CF0,40
^FO100,960^FDCtr. X34B-1^FS
^FO100,1010^FDREF1 F00B47^FS
^FO100,1060^FDREF2 BL4H8^FS
^CF0,190
^FO470,955^FDCA^FS

^XZ
"""

# You can create and edit labels here: https://labelary.com/viewer.html
printer_params = {}
try:
    json_path = Path(__file__).parents[4] / 'pywiliot-testers' / 'wiliot_testers' / 'association_tester' / 'configs' / 'r2r_printer_params.json'
    with open(json_path, 'r') as f:
        printer_params = json.load(f)
except Exception as e:
    print(f"Failed to load printer params from json: {e}")
p = ZebraPrinter(**printer_params)

# print simple label given as ZPL
p.print_label(label)

# print label from label_format_path and label_content_path
s = p.print_next_label()

# save example label to file
with open(printer_params['label_format_path'][:-4] + '_example.txt', 'w') as f:
    f.write(s)



#  """
#    Copyright (c) 2016- 2022, Wiliot Ltd. All rights reserved.
#
#    Redistribution and use of the Software in source and binary forms, with or without modification,
#     are permitted provided that the following conditions are met:
#
#       1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#
#       2. Redistributions in binary form, except as used in conjunction with
#       Wiliot's Pixel in a product or a Software update for such product, must reproduce
#       the above copyright notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the distribution.
#
#       3. Neither the name nor logo of Wiliot, nor the names of the Software's contributors,
#       may be used to endorse or promote products or services derived from this Software,
#       without specific prior written permission.
#
#       4. This Software, with or without modification, must only be used in conjunction
#       with Wiliot's Pixel or with Wiliot's cloud service.
#
#       5. If any Software is provided in binary form under this license, you must not
#       do any of the following:
#       (a) modify, adapt, translate, or create a derivative work of the Software; or
#       (b) reverse engineer, decompile, disassemble, decrypt, or otherwise attempt to
#       discover the source code or non-literal aspects (such as the underlying structure,
#       sequence, organization, ideas, or algorithms) of the Software.
#
#       6. If you create a derivative work and/or improvement of any Software, you hereby
#       irrevocably grant each of Wiliot and its corporate affiliates a worldwide, non-exclusive,
#       royalty-free, fully paid-up, perpetual, irrevocable, assignable, sublicensable
#       right and license to reproduce, use, make, have made, import, distribute, sell,
#       offer for sale, create derivative works of, modify, translate, publicly perform
#       and display, and otherwise commercially exploit such derivative works and improvements
#       (as applicable) in conjunction with Wiliot's products and services.
#
#       7. You represent and warrant that you are not a resident of (and will not use the
#       Software in) a country that the U.S. government has embargoed for use of the Software,
#       nor are you named on the U.S. Treasury Department’s list of Specially Designated
#       Nationals or any other applicable trade sanctioning regulations of any jurisdiction.
#       You must not transfer, export, re-export, import, re-import or divert the Software
#       in violation of any export or re-export control laws and regulations (such as the
#       United States' ITAR, EAR, and OFAC regulations), as well as any applicable import
#       and use restrictions, all as then in effect
#
#     THIS SOFTWARE IS PROVIDED BY WILIOT "AS IS" AND "AS AVAILABLE", AND ANY EXPRESS
#     OR IMPLIED WARRANTIES OR CONDITIONS, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED
#     WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, NONINFRINGEMENT,
#     QUIET POSSESSION, FITNESS FOR A PARTICULAR PURPOSE, AND TITLE, ARE DISCLAIMED.
#     IN NO EVENT SHALL WILIOT, ANY OF ITS CORPORATE AFFILIATES OR LICENSORS, AND/OR
#     ANY CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
#     OR CONSEQUENTIAL DAMAGES, FOR THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES,
#     FOR ANY LOSS OF USE OR DATA OR BUSINESS INTERRUPTION, AND/OR FOR ANY ECONOMIC LOSS
#     (SUCH AS LOST PROFITS, REVENUE, ANTICIPATED SAVINGS). THE FOREGOING SHALL APPLY:
#     (A) HOWEVER CAUSED AND REGARDLESS OF THE THEORY OR BASIS LIABILITY, WHETHER IN
#     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE);
#     (B) EVEN IF ANYONE IS ADVISED OF THE POSSIBILITY OF ANY DAMAGES, LOSSES, OR COSTS; AND
#     (C) EVEN IF ANY REMEDY FAILS OF ITS ESSENTIAL PURPOSE.
#  """

# keys : macro name:
## duration [seconds]
## command:
###   gateway commands
###
##
"""
keys are the macro name
each macro contains:
    * command:
    ** if starts with !, it is gw command o send
    ** user_event, it will trigger user event with the specified text under 'values' key
    ** save log, it will save the log at the specified location under 'values' key
    * wait : time to wait in seconds after the command is done

"""

macros = {
    "update_version_macro": [{"command": "update_version", 'values': r'', "wait": 1}],
    "example_macro": [
        {"command": "!version", "wait": 3},
        {"command": "user_event", 'values': 'Set tag in initial location', "wait": 1},
        {"command": "!output_power pos3dBm", "wait": 5},
        {"command": "user_event", 'values': 'Pick up tag', "wait": 1},
        {"command": "!gateway_app 37 15 5 18", "wait": 3},
        {"command": "!cancel", "wait": 1},
        {"command": "!version", "wait": 1},
        {"command": "save_log", 'values': r'~/Downloads/output.csv', "wait": 1},

    ],
    "sub1g_calibration": [
        {"command": "!set_sub_1_ghz_power 29", "wait": 0},
        {"command": "!set_sub_1_ghz_energizing_frequency 915000", "wait": 0},
        {"command": "!beacons_backoff 0", "wait": 0},
        {"command": "user_event", 'values': 'Set new config', "wait": 0},
        {"command": "!gateway_app 37 15 5 50", "wait": 90},  # test for 90 seconds
        {"command": "!cancel", "wait": 1},
        {"command": "user_event", 'values': 'Stop gw app and eneter into brownout', "wait": 300},
        {"command": "save_log", 'values': r'~/Downloads/output.csv', "wait": 0},
    ],
    "toggle_rx_channel": [
        {"cyclic": True},
        {"command": "!scan_ch 37 37", "wait": 1},
        {"command": "!scan_ch 0 0", "wait": 1},
    ],
    "toggle_rx_advertising_channels": [
        {"cyclic": True},
        {"command": "!scan_ch 37", "wait": 1},
        {"command": "!scan_ch 38", "wait": 1},
        {"command": "!scan_ch 39", "wait": 1},
    ],
    "send_pulse": [
        {"cyclic": True},
        {"command": "!cmd_gpio SEND 1 P100 pulse 1 500", "wait": 1},
    ],

}



import json
import time
import traceback
from datetime import datetime
from PIL import Image, ImageTk
import os
from wiliot_api import ManufacturingClient, WiliotCloudError
from wiliot_tools.utils.wiliot_gui.wiliot_gui import WiliotGui
from yoctopuce.yocto_api import YAPI, YRefParam
from yoctopuce.yocto_humidity import YHumidity
from yoctopuce.yocto_temperature import YTemperature
import requests
from zebra import Zebra
from wiliot_core import GetApiKey
import pytz

def get_israel_time(retries=3):
    return str(datetime.now(tz=pytz.timezone("Asia/Jerusalem"))).split(".")[0]
    # primary_url = "https://timeapi.io/api/Time/current/zone?timeZone=Asia/Jerusalem"
    # backup_url = "http://worldtimeapi.org/api/timezone/Asia/Jerusalem"
    #
    # for attempt in range(retries):
    #     try:
    #         response = requests.get(primary_url)
    #         if response.status_code == 200:
    #             data = response.json()
    #             current_time = data['dateTime']
    #             return current_time.split(".")[0].replace("T", " ")
    #         else:
    #             print(f"Primary API returned unexpected status code: {response.status_code}")
    #     except requests.exceptions.RequestException as ex:
    #         print(f"Primary API attempt {attempt + 1} failed: {ex}")
    #
    #     try:
    #         response = requests.get(backup_url)
    #         if response.status_code == 200:
    #             data = response.json()
    #             current_time = data['datetime']
    #             return current_time.split(".")[0].replace("T", " ")
    #         else:
    #             print(f"Backup API returned unexpected status code: {response.status_code}")
    #     except requests.exceptions.RequestException as ex:
    #         print(f"Backup API attempt {attempt + 1} failed: {ex}")
    #
    # print("Failed to retrieve Israel time after multiple attempts.")
    # return None


def title_button(title, width):
    width = width - len(title)
    title = ' ' * (width // 2) + title + ' ' * (width // 2)
    if width % 2 == 1:
        title += ' '
    return title


def get_manufacturing_client(env):
    owner_id = '852213717688'
    if env == 'test':
        owner_id = 'wiliot-ops'
    key_obj = GetApiKey(env=env, owner_id=owner_id)
    key = key_obj.get_api_key()
    if key:
        try:
            return ManufacturingClient(env=env, api_key=key)
        except:
            print("Unable to create an instance of ManufacturingClient.")
            return
    else:
        print("Please enter an API key in the config.json file to enable log uploads to the cloud.")
        return


def upload_json_to_s3(client, data, name, directory):
    if client is None:
        print("Please enter an API key in the config.json file to enable log uploads to the cloud.")
        return
    timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
    filename = f"{name}_{timestamp}.json"
    with open(filename, 'w') as json_file:
        json.dump(data, json_file)
    try:
        res = client.partner_upload_file_to_s3(file_path=filename, directory=directory)
        print("Log was uploaded to the cloud successfully!")
        print("Response:", res)
    except WiliotCloudError:
        print("Log upload to the cloud failed!")
    except Exception as e:
        print("Log upload to the cloud failed due to exception", e)
    os.remove(filename)


def format_date(date_str):
    try:
        date_obj = datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")
        return date_obj.strftime("%d%m%Y")
    except ValueError:
        return None


def print_label(label, printer_name):
    try:
        zebra_printer = Zebra()
        zebra_printer.connection = 'usb'
        zebra_printer.device = 'USB001'
        zebra_printer.setqueue(printer_name)
        zebra_printer.output(label)
        return True
    except Exception:
        print("Failed to print label!")
        traceback.print_exc()
        return False


class YoctoSensor(object):
    def __init__(self):
        self.sensor = None
        errmsg = YRefParam()
        # Setup the API to use local USB devices
        if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
            raise Exception('yocto temperature sensor got init error: {}'.format(errmsg.value))

    def connect(self, target='any'):
        if target == 'any':
            # retrieve any temperature sensor
            self.sensorHum = YHumidity.FirstHumidity()
            self.sensorTemp = YTemperature.FirstTemperature()

        elif target == '':
            print('specified invalid target')
            return False
        else:
            self.sensorHum = YHumidity.FindHumidity(target + '.humidity')
            self.sensorTemp = YTemperature.FindTemperature(target + '.temperature')

        if self.sensorHum is None or self.sensorTemp is None or self.get_sensor_name() == 'unresolved':
            print('Failed to connect to sensor')
            return False

        else:
            return True

    def get_sensor_name(self):
        if self.sensor is None:
            # print('no sensor is connected. try to call connect() first')
            return ''

        sensor_str = self.sensor.describe()
        name_str = sensor_str.split('=')[1].split('.')[0]
        return name_str

    def get_humidity(self):
        if self.sensorHum is None:
            # print('sensor is not connected. try to call connect() first')
            return None
        if not (self.sensorHum.isOnline()):
            # print('sensor is not connected or disconnected during run')
            return None

        return self.sensorHum.get_currentValue()

    def get_temperature(self):
        if self.sensorTemp is None:
            # print('sensor is not connected. try to call connect() first')
            return None
        if not (self.sensorTemp.isOnline()):
            # print('sensor is not connected or disconnected during run')
            return None

        return self.sensorTemp.get_currentValue()

    @staticmethod
    def exit_app():
        YAPI.FreeAPI()


class WiliotGuiCustomized(WiliotGui):
    def on_close(self):
        self.parent.save_history()
        self.exit_gui()

    def set_parent(self, parent):
        self.parent = parent



empty_page = {
        "assembly_id": "1",
        "operator_name": "-",
        "inlay": "-",
        "assembled_reel_width": "",
        "#_of_lanes_in_antenna_design": "-",
        "assembled_reel_count": "-",
        "warning": "",
        "total_dies_qty_assembled": "0",
        "space": "",
        "tabs_1) wafer_qr_code": "",
        "tabs_1) temperature": "",
        "tabs_1) humidity": "",
        "tabs_1) date": "",
        "tabs_2) wafer_qr_code": "",
        "tabs_2) wafer_dies_qty_assembled": "",
        "tabs_2) temperature": "",
        "tabs_2) humidity": "",
        "tabs_2) date": "",
        "tabs_3) wafer_qr_code": "",
        "tabs_3) temperature": "",
        "tabs_3) humidity": "",
        "tabs_3) date": "",
        "tabs_4) wafer_qr_code": "",
        "tabs_4) wafer_dies_qty_assembled": "",
        "tabs_4) temperature": "",
        "tabs_4) humidity": "",
        "tabs_4) date": "",
        "tabs_5) wafer_qr_code": "",
        "tabs_5) temperature": "",
        "tabs_5) humidity": "",
        "tabs_5) date": "",
        "tabs_6) wafer_qr_code": "",
        "tabs_6) wafer_dies_qty_assembled": "",
        "tabs_6) temperature": "",
        "tabs_6) humidity": "",
        "tabs_6) date": "",
        "assembly_label": "",
        "wafers": "",
        "output": "",
    }

default_config = {
        "operator_name": ['Adrian', 'Liel', 'Yaniv', 'Yael'],
        "inlay": [117, 169],
        "#_of_lanes_in_antenna_design": [4, 5, 6],
        "assembled_reel_count": [1, 2, 3, 4, 5, 6],
        "printer_name": "",
        "inlay_rows": {
                "117": 5,
                "169": 4
            }
}


import os
import subprocess
from datetime import datetime
import platform
from PIL import Image, ImageTk
import json
try:
    import customtkinter as ctk
except Exception as e:
    print(f'could not import tkinter: {e}')
import pandas as pd
from pathlib import Path

FILE_PATH = Path(__file__).parent.resolve()


def col_width():
    with open(FILE_PATH / "resources" / "columns_width.json", "r") as json_file:
        cols_width = json.load(json_file)
    return cols_width


def show_img(frame):
    try:
        image = Image.open(FILE_PATH / "resources" /" wiliot.png")
        image = image.resize((150, 75))
        tk_image = ImageTk.PhotoImage(image)
        image_label = ctk.CTkLabel(frame, text="", image=tk_image)
        image_label.image = tk_image
        image_label.place(x=1020, y=0)
    except:
        print("image was not found")


def can_be_int(value):
    try:
        val = int(value)
        return val < 10000
    except ValueError:
        return False


def create_tags_df(request_df, seperator='T'):
    rows = []
    for i, row in request_df.iterrows():
        for x in range(int(row['First Tag']), int(row['Last Tag']) + 1):
            rows.append(row['Ex. ID Prefix'] + seperator + str(x).zfill(4))
    rows = set(rows)
    return pd.DataFrame([[row] for row in rows], columns=['tagId']).sort_values('tagId')


def verify_file(df):
    if not all((df['Ex. ID Prefix'].str.len() >= 3)):
        raise ValueError(f"Not all values for column 'Ex. ID Prefix' are correct\n"
                         "please verify that all values at least 3 letters")

    if not df['First Tag'].apply(can_be_int).all():
        raise ValueError(f"Not all values for column 'First Tag' are numbers and under 10000")

    if not df['Last Tag'].apply(can_be_int).all():
        raise ValueError(f"Not all values for column 'Last Tag' are numbers and under 10000")

    if not df['To Owner'].apply(lambda x: x != "" and "." not in str(x) and "+" not in str(x)).all():
        raise ValueError(f"To Owner values are invalid.")

    for i, row in df.iterrows():
        if str(row['First Tag']).zfill(4) > str(row['Last Tag']).zfill(4):
            raise ValueError(f"First Tag is bigger than Last Tag\n"
                             f"({row['Ex. ID Prefix']}, {row['First Tag']}, {row['Last Tag']})")


def open_csv_with_excel(csv_file_path):
    if not os.path.exists(csv_file_path):
        raise Exception(f"The file {csv_file_path} does not exist.")
    timestamp_now_secs = datetime.now().strftime("%Y%m%d%H%M%S")
    new_path = csv_file_path[:-4]+timestamp_now_secs+".csv"
    pd.read_csv(csv_file_path, index_col=None).to_csv(new_path, index=False)
    try:
        if platform.system() == "Windows":
            subprocess.Popen(["start", "excel", new_path], shell=True)
        elif platform.system() == "Darwin":
            subprocess.call(["open", "-a", "Microsoft Excel", new_path])
        else:
            print("This function is not supported on this operating system.")
    except FileNotFoundError:
        raise Exception("Microsoft Excel is not found. Please make sure Excel is installed on your computer.")
    except Exception as e:
        print(f"An error occurred: {e}")



import json
import os
import time
try:
    import tkinter as tk
except Exception as e:
    print(f'could not import tkinter: {e}')
from datetime import datetime

from wiliot_tools.tadbik_label_printers.conversion.data.default_values import default_config
from wiliot_tools.utils.wiliot_gui.wiliot_gui import WiliotGui
from wiliot_tools.tadbik_label_printers.label_printer_tools import *


class ConversionLabelPrinter:
    def __init__(self, env="prod"):
        self.sensor = YoctoSensor()
        self.sensor.connect()
        self.counter = 0
        self.last_batch_id = None
        self.history = self.read_history()
        self.config = self.read_config()
        self.client = get_manufacturing_client(env)
        self.init_widgets()
        self.app.layout.title("conversion Label Printer")
        self.events()
        self.app.add_recurrent_function(cycle_ms=1000, function=self.recurrent)
        self.load_batch_data()
        self.app.set_parent(self)
        self.inputs_security()
        self.app.run()

    def init_widgets(self):
        main_dict = {}
        main_dict["operator_name"] = ['-'] + self.config.get('operator_name', [])
        main_dict["product_configuration"] = ['-'] + self.config.get('product_configuration', ['Cardboard', 'Plastic', 'Light'])
        main_dict["inlay"] = ['-'] + self.config.get('inlay', [])
        main_dict["start_date"] = ''
        main_dict["assembled_reel_qr_code"] = ''
        main_dict["lanes_ids"] = ''
        main_dict["number_of_lanes"] = ['-'] + self.config.get('number_of_lanes', [])
        main_dict["converted_reels_serial_number"] = list(
            range(1, int(self.config.get('converted_reels_serial_number', 200)) + 1))
        main_dict["seq_assy_reel"] = list(range(1, int(self.config.get('seq_assy_reel', 100)) + 1))
        main_dict["humidity"] = ''
        main_dict["temperature_input"] = ''
        main_dict["print_label"] = {'value': title_button('Print Label', 36), 'widget_type': 'button', 'columnspan': 3}
        main_dict["assembled_reel_conversion_completion"] = {
            'value': title_button('Assembled Reel conversion Completion', 36), 'widget_type': 'button', 'columnspan': 3}
        main_dict["output"] = {'value': '', 'widget_type': 'label', 'options': {'fg': 'red'}, 'columnspan': 3}
        main_dict["end_date"] = ''
        main_dict["total_batch_conversion_tags_qty"] = ''
        main_dict["batch_conversion_completion"] = {'value': title_button('Batch conversion Completion', 36),
                                                    'widget_type': 'button', 'columnspan': 3}
        main_dict["warning"] = {'value': '', 'widget_type': 'label', 'options': {'fg': 'red'}, 'columnspan': 3}

        main_dict = {k: {'value': v} if not isinstance(v, dict) else v for k, v in main_dict.items()}

        main_dict["humidity"] = [
            {"input": {'value': '', 'widget_type': 'entry',
                       'text': 'Humidity'}},
            {"record": {'value': 'Record Humidity & Temperature', 'widget_type': 'button'}}
        ]
        main_dict["temperature_input"] = {'value': '', 'text': 'Temperature'}
        main_dict["start_date"] = [
            {"batch_conversion_start_date": {'value': '', 'widget_type': 'entry',
                                             'text': 'Batch conversion start date'}},
            {"record_start_date": {'value': 'Record Date', 'widget_type': 'button'}}
        ]
        main_dict["end_date"] = [
            {"batch_conversion_end_date": {'value': '', 'widget_type': 'entry',
                                           'text': 'Batch conversion end date'}},
            {"record_end_date": {'value': 'Record Date', 'widget_type': 'button'}}
        ]
        main_dict["lanes_ids"] = {'value': '', 'text': 'Lanes IDs (e.g. A,B,C)'}


        self.app = WiliotGuiCustomized(main_dict, full_screen=True, theme='wiliot', do_button_config=False)

    def events(self):
        self.app.add_event(widget_key='humidity_record', event_type='button', command=self.update_sensors)
        self.app.add_event(widget_key='print_label', event_type='button', command=self.print_label)
        self.app.add_event(widget_key='assembled_reel_conversion_completion', event_type='button',
                           command=self.assembled_reel_conversion_completion)
        self.app.add_event(widget_key='batch_conversion_completion', event_type='button',
                           command=self.batch_conversion_completion)

        self.app.add_event(widget_key='inlay', event_type='<<ComboboxSelected>>', command=self.update_inlay)
        self.app.add_event(widget_key='start_date_record_start_date', event_type='button',
                           command=lambda: self.app.update_widget(widget_key='start_date_batch_conversion_start_date',
                                                                  new_value=get_israel_time()))
        self.app.add_event(widget_key='end_date_record_end_date', event_type='button',
                           command=lambda: self.app.update_widget(widget_key='end_date_batch_conversion_end_date',
                                                                  new_value=get_israel_time()))

    def update_sensors(self):
        temp = self.sensor.get_temperature()
        if temp is None:
            temp = 'Not Connected'
        hum = self.sensor.get_humidity()
        if hum is None:
            hum = 'Not Connected'
        self.app.update_widget(widget_key=f'temperature_input', new_value=temp)
        self.app.update_widget(widget_key=f'humidity_input', new_value=hum)

    def inputs_security(self):
        dropdowns = ['operator_name', 'inlay', 'seq_assy_reel', 'number_of_lanes', 'converted_reels_serial_number']
        for dd in dropdowns:
            self.app.widgets[dd].config(state='readonly')

        def only_letters_and_commas(input_value):
            if input_value == "" or all(
                    ('A' <= char.upper() <= 'F' and i % 2 == 0) or (char in (',', ',') and i % 2 == 1) for i, char in
                    enumerate(input_value)):
                return True
            return False

        vcmd = (self.app.class_tk.register(only_letters_and_commas), '%P')
        self.app.widgets['lanes_ids'].config(validate='key', validatecommand=vcmd)

        def only_numbers(input_value):
            if input_value == "" or all(char.isdigit() for i, char in enumerate(input_value)):
                return True
            return False

        vcmd = (self.app.class_tk.register(only_numbers), '%P')
        self.app.widgets['total_batch_conversion_tags_qty'].config(validate='key', validatecommand=vcmd)

    def recurrent(self):
        if self.counter % 10 == 9:
            self.save_history(output_message=False)
        self.counter += 1

    def read_history(self):
        try:
            with open(os.path.join('data', 'conversion_history.json'), 'r') as file:
                file = json.load(file)
            return file
        except FileNotFoundError:
            return {get_israel_time(): {}}

    def read_config(self):
        try:
            with open('config.json', 'r') as file:
                file = json.load(file)
            return file
        except FileNotFoundError:
            with open('config.json', 'w') as file:
                json.dump(default_config, file, indent=4)
            return default_config

    def save_page(self, output_message=True):
        values = self.app.get_all_values()
        batch_id = max(self.history.keys())
        self.history[batch_id] = values
        if output_message:
            print("Saved current page successfully.")

    def save_history(self, output_message=True):
        try:
            self.save_page(output_message)
            with open(os.path.join('data', 'conversion_history.json'), 'w') as file:
                json.dump(self.history, file, indent=4)
            if output_message:
                print("History saved successfully.")
        except Exception as e:
            print(f"An error occurred while saving history: {e}")

    def load_batch_data(self):
        batch_id = max(self.history.keys())
        data = self.history.get(batch_id, {})
        for key, value in data.items():
            self.app.update_widget(widget_key=key, new_value=value)

    def assembled_reel_conversion_completion(self):
        self.app.update_widget(widget_key='assembled_reel_qr_code', new_value='')
        current_seq_assy_reel = self.app.get_all_values().get('seq_assy_reel')
        if current_seq_assy_reel != '-':
            next_reel = int(current_seq_assy_reel) + 1
            self.app.update_widget(widget_key='seq_assy_reel', new_value=str(next_reel))
        else:
            self.app.update_widget(widget_key='seq_assy_reel', new_value='1')
        self.app.update_widget(widget_key='converted_reels_serial_number', new_value='1')

    def batch_conversion_completion(self):
        self.history[get_israel_time()] = {}
        self.app.update_widget(widget_key='end_date_batch_conversion_end_date', new_value='')
        self.app.update_widget(widget_key='seq_assy_reel', new_value='1')
        self.app.update_widget(widget_key='converted_reels_serial_number', new_value='1')

        self.app.update_widget(widget_key='operator_name', new_value='-')
        self.app.update_widget(widget_key='inlay', new_value='-')
        self.app.update_widget(widget_key='start_date_batch_conversion_start_date', new_value='')
        self.app.update_widget(widget_key='total_batch_conversion_tags_qty', new_value='')
        self.app.update_widget(widget_key='lanes_ids', new_value='')
        self.app.update_widget(widget_key='number_of_lanes', new_value='-')
        self.app.update_widget(widget_key='assembled_reel_qr_code', new_value='')
        self.app.update_widget(widget_key='output', new_value='')
        self.app.update_widget(widget_key='warning', new_value='')
        self.app.update_widget(widget_key='humidity_input', new_value='')
        self.app.update_widget(widget_key='temperature_input', new_value='')
        self.save_history()

    def update_inlay(self, _):
        pass

    def verify_values(self, values):
        error_messages = []

        if values.get('operator_name') == '-':
            error_messages.append('Please select an operator name.')

        if values.get('inlay') == '-':
            error_messages.append('Please select an inlay.')

        batch_start_date = format_date(values['start_date_batch_conversion_start_date'])
        if batch_start_date is None:
            error_messages.append('Batch conversion start date is not a valid date format.')

        assembled_reel_qr_code = values.get('assembled_reel_qr_code')
        if not assembled_reel_qr_code:
            error_messages.append('Please scan the Assembled Reel QR Code.')
        elif assembled_reel_qr_code.split("_")[0] != values.get('inlay'):
            error_messages.append('The scanned QR code does not match the selected inlay.')

        lanes_ids = values['lanes_ids'].replace(' ', '').rstrip(',').split(',')
        if not lanes_ids or lanes_ids == ['']:
            error_messages.append('Please enter Lane IDs.')

        number_of_lines = values.get('number_of_lanes')
        if number_of_lines == '-':
            error_messages.append('Please select the number of lanes.')

        if not error_messages and len(lanes_ids) != int(number_of_lines):
            error_messages.append(
                f'The number of Lane IDs provided ({len(lanes_ids)}) does not match the selected number of lanes ({number_of_lines}).')

        if values.get('humidity_input') == '' or values.get('temperature_input') == '':
            error_messages.append('Please record the humidity and temperature.')

        if error_messages:
            full_error_message = error_messages[0]
            self.app.update_widget(widget_key='output', new_value=full_error_message, color='red')
            return False
        else:
            return True

    def print_label(self):
        values = self.app.get_all_values()
        if not self.verify_values(values):
            return

        self.app.update_widget(widget_key='humidity_input', new_value='')
        self.app.update_widget(widget_key='temperature_input', new_value='')

        inlay = values['inlay']
        seq_assy_reel = values['seq_assy_reel']
        lanes_ids = values['lanes_ids'].replace(' ', '').rstrip(',').split(',')
        conv_date = format_date(values['start_date_batch_conversion_start_date'])
        converted_reels_serial_number = values['converted_reels_serial_number']
        product_config = values['product_configuration']

        for lane_id in lanes_ids:
            label_name = f"{inlay}_{seq_assy_reel}_{lane_id.upper()}_{conv_date}_{converted_reels_serial_number}"
            label = f"""
                     ^XA
                    ^FO0,0^GFA,3024,3024,27,,::::::::::::::::::::::gK01FCQ03F8,::::gK01FCP0F3F8,gK01FCO03IF8,gK01FCO07IF8,:gK01FCO0F0FF8,gK01FCO0F07F8,gK01FCO0E07F8,:gK01FCO0F07F8,gK01FCO0F8FF8,gK01FCO07IF8,O0FEI0FE001FC07F01FC07FL03IF8,O0FEI0FE001FC07F01FC07FL01JFC,O0FEI0FE001FC07F01FC07FJ0EI03FFC,O0FEI0FE001FC07F01FC07FI0FFE003FFC,O0FEI0FE001FC07F01FC07F003IF003FFC,O0FEI0FE001FC07F01FC07F007IFC03FFC,O0FEI0FE001FC07F01FC07F00JFE03FFC,O0FEI0FE001FCJ01FCJ01KF03FFC,O0FEI0FE001FCJ01FCJ03KF03F8,O0FEI0FE001FCJ01FCJ03KF83F8,O0FEI0FE001FC07F01FC07F07FC07F83F8,O0FEI0FE001FC07F01FC07F07F803FC3F8,O0FEI0FE001FC07F01FC07F0FF001FC3F8,O0FEI0FE001FC07F01FC07F0FE001FC3F8,O0FEI0FE001FC07F01FC07F0FE001FE3F8,::O0FEI0FE001FC07F01FC07F0FE001FC3F8,O0FEI0FE001FC07F01FC07F0FF001FC3F8,O0FEI0FE001FC07F01FC07F07F003FC3F8,O0FEI0FE001FC07F01FC07F07F807FC3F8,O0FEI0FE001FC07F01FC07F07FE1FF83F8,O0FEI0FE001FC07F01FC07F03KF83F8,O0FEI0FE001FC07F01FC07F01KF03F8,O0FEI0FE001FC07F01FC07F01JFE03F8,O0FEI0FE001FC07F01FC07F00JFC03F8,O0FEI0FE001FC07F01FC07F003IF803F8,O0FEI0FE001FC07F01FC07F001FFE003F8,O0FE001FE001FC07F01FC07FI03F8003F8,O0FF001FE001FC07F01FC07FN03F8,O07F003FF003FC07F01FC07FN03FC,O07F803FF807F807F01FC07FN01FE,O07FE0IFC0FF807F01FC07FN01FF03,O03PF007F01FC07FO0JF8,O01PF007F01FC07FO0JFC,O01OFE007F01FC07FO07IFE,P0JFEJFC007F01FC07FO03JF,P07IF87IF8007F01FC07FO01IFE,P01IF03FFEI07F01FC07FP07FFC,Q07FC007F8I07F01FC07FP01FE,,::::::::::::::::::::::::::::::::^FS
                    ^LH0,0
                            ^FO450,15^BQN,2,5,L
                    ^FDMA,{label_name}^FS
                    ^FO370,200^A0N,30,30^FD{label_name}^FS
                    ^FO30,200^A0N,40,40^FDAFTER CONV^FS
                    ^FO30,125^A0N,28,28^FDRow#{lane_id.upper()}^FS
                    ^XZ
                    """
            if product_config == 'Plastic':
                reel_name = self.get_reel_name()
                reel_row_name = f"{reel_name}_{lane_id.upper()}"
                reel_name_label = f"""
                     ^XA
                    ^FO0,0^GFA,3024,3024,27,,::::::::::::::::::::::gK01FCQ03F8,::::gK01FCP0F3F8,gK01FCO03IF8,gK01FCO07IF8,:gK01FCO0F0FF8,gK01FCO0F07F8,gK01FCO0E07F8,:gK01FCO0F07F8,gK01FCO0F8FF8,gK01FCO07IF8,O0FEI0FE001FC07F01FC07FL03IF8,O0FEI0FE001FC07F01FC07FL01JFC,O0FEI0FE001FC07F01FC07FJ0EI03FFC,O0FEI0FE001FC07F01FC07FI0FFE003FFC,O0FEI0FE001FC07F01FC07F003IF003FFC,O0FEI0FE001FC07F01FC07F007IFC03FFC,O0FEI0FE001FC07F01FC07F00JFE03FFC,O0FEI0FE001FCJ01FCJ01KF03FFC,O0FEI0FE001FCJ01FCJ03KF03F8,O0FEI0FE001FCJ01FCJ03KF83F8,O0FEI0FE001FC07F01FC07F07FC07F83F8,O0FEI0FE001FC07F01FC07F07F803FC3F8,O0FEI0FE001FC07F01FC07F0FF001FC3F8,O0FEI0FE001FC07F01FC07F0FE001FC3F8,O0FEI0FE001FC07F01FC07F0FE001FE3F8,::O0FEI0FE001FC07F01FC07F0FE001FC3F8,O0FEI0FE001FC07F01FC07F0FF001FC3F8,O0FEI0FE001FC07F01FC07F07F003FC3F8,O0FEI0FE001FC07F01FC07F07F807FC3F8,O0FEI0FE001FC07F01FC07F07FE1FF83F8,O0FEI0FE001FC07F01FC07F03KF83F8,O0FEI0FE001FC07F01FC07F01KF03F8,O0FEI0FE001FC07F01FC07F01JFE03F8,O0FEI0FE001FC07F01FC07F00JFC03F8,O0FEI0FE001FC07F01FC07F003IF803F8,O0FEI0FE001FC07F01FC07F001FFE003F8,O0FE001FE001FC07F01FC07FI03F8003F8,O0FF001FE001FC07F01FC07FN03F8,O07F003FF003FC07F01FC07FN03FC,O07F803FF807F807F01FC07FN01FE,O07FE0IFC0FF807F01FC07FN01FF03,O03PF007F01FC07FO0JF8,O01PF007F01FC07FO0JFC,O01OFE007F01FC07FO07IFE,P0JFEJFC007F01FC07FO03JF,P07IF87IF8007F01FC07FO01IFE,P01IF03FFEI07F01FC07FP07FFC,Q07FC007F8I07F01FC07FP01FE,,::::::::::::::::::::::::::::::::^FS
                    ^LH0,0
                            ^FO300,15^BQN,2,5,L
                    ^FDMA,{reel_row_name}^FS
                    ^FO290,150^A0N,40,40^FD{reel_row_name}^FS
                    ^XZ
                    """
                print(f"Printing label: {reel_row_name} \n {reel_name_label}")
                if print_label(reel_name_label, self.config.get("printer_name")):
                    with open('reel_name.json', 'w') as file:
                        json.dump({'reel_name': reel_name}, file, indent=4)

            if print_label(label, self.config.get("printer_name")):
                self.app.update_widget(widget_key='output',
                                       new_value=f'Labels successfully sent to printer on {datetime.now()}',
                                       color='green')
            else:
                self.app.update_widget(widget_key='output', new_value='Failed to connect to the printer.', color='red')
            print(f"Printing label: {label_name} \n {label}")
        next_serial_number = int(converted_reels_serial_number) + 1
        self.app.update_widget(widget_key='converted_reels_serial_number', new_value=str(next_serial_number))
        label_names = f"{inlay}_{seq_assy_reel}_{lanes_ids[0].upper()}-{lanes_ids[-1].upper()}_{conv_date}_{converted_reels_serial_number}"
        upload_json_to_s3(self.client, values, label_names, 'conversion')

    def get_last_reel_name(self):
        try:
            with open('reel_name.json', 'r') as file:
                file = json.load(file)
            return file['reel_name']
        except FileNotFoundError:
            with open('reel_name.json', 'w') as file:
                json.dump({'reel_name': '0ZZZ'}, file, indent=4)
            return '9ZZZ'

    @staticmethod
    def inc36(s):
        digits = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        s = list(s)
        i = len(s) - 1
        carry = 1
        while i >= 0 and carry:
            v = digits.index(s[i]) + carry
            s[i] = digits[v % 36]
            carry = v // 36
            i -= 1
        if carry:
            s.insert(0, digits[carry])
        return ''.join(s)

    def get_reel_name(self):
        reel_name = self.get_last_reel_name()
        reel_name = self.inc36(reel_name)
        return reel_name




if __name__ == '__main__':
    ConversionLabelPrinter()



import configparser
import os
try:
    from tkinter import filedialog
except Exception as e:
    print(f'could not import tkinter: {e}')
from CTkMessagebox import CTkMessagebox
from owner_change_utils import *
from datetime import datetime

class MultiReelRequestPage(ctk.CTkFrame):

    def __init__(self, parent, show_page_callback):
        super().__init__(parent)
        self.parent = parent
        self.show_page_callback = lambda: show_page_callback(self)
        self.tree = None
        self.parent.attached_df = None
        self.cbs = []
        label = ctk.CTkLabel(self, text="Create Request (Multiple Reels)", font=("Arial", 28, "bold"))
        label.pack(pady=50)
        show_img(self)
        start_y = self.start_y = 30
        label = ctk.CTkLabel(self, text="API Key:")
        label.place(x=50, y=60 + start_y)

        self.api_key_entry = ctk.CTkEntry(self, width=200, show="*")
        self.api_key_entry.place(x=140, y=60 + start_y)
        self.api_key_entry.insert(0, self.parent.api_key)

        label = ctk.CTkLabel(self, text="From Owner:")
        label.place(x=50, y=110 + start_y)

        # self.from_owner_entry = combobox = ctk.CTkComboBox(master=self,
        #                                                    values=["wiliot-ops", "108502199256 (tadbik-mnf)"],
        #                                                    width=200, justify="center")

        self.from_owner_entry = ctk.CTkEntry(self, width=200)
        self.from_owner_entry.place(x=140, y=110 + start_y)
        self.from_owner_entry.insert(0, 'wiliot-ops')

        self.file_attach_label = ctk.CTkLabel(self, text="")
        self.file_attach_label.place(x=140, y=160 + start_y)
        self.cloud_dest_entry = ctk.CTkComboBox(master=self,values=["AWS", "WMT"],width=200, justify="center")
        self.cloud_dest_entry.place(x=140, y= 20 + start_y)
        self.cloud_dest_entry.set('WMT')
        btn = ctk.CTkButton(self, text="Attach File", command=self.browse_file, width=70)
        btn.place(x=50, y=160 + start_y)

        for i in range(4):
            combobox = ctk.CTkComboBox(master=self,
                                       values=["-"],
                                       command=self.combobox_callback,
                                       width=200, justify="center")
            combobox.place(x=90 + 275 * i, y=260 + start_y)
            self.cbs.append(combobox)

        self.tree = self.parent.create_tree_view(self, pd.DataFrame(columns=self.parent.cols), height=15)
        self.tree.place(x=50, y=300 + start_y)

        btn = ctk.CTkButton(self, text="Back", command=self.show_page_callback, width=120)
        btn.place(x=50, y=self.parent.H - 50)

        btn = ctk.CTkButton(self, text="Send Request", command=self.send_request, width=120)
        btn.place(x=self.parent.W / 2 - 60, y=self.parent.H - 35)

        self.display_attached_file()

    def combobox_callback(self, event):
        self.update_df()

    def select_closest_col(self):
        for cb in self.cbs:
            cb.set('-')
        for col in self.parent.attached_df.columns:
            lo_col = col.lower()
            if 'prefix' in lo_col:
                self.cbs[0].set(col)
            if 'first' in lo_col or 'start' in lo_col or 'init' in lo_col:
                self.cbs[1].set(col)
            if 'last' in lo_col or 'end' in lo_col:
                self.cbs[2].set(col)
            if 'to' in lo_col and 'owner' in lo_col:
                self.cbs[3].set(col)

    def send_request(self):
        try:
            failed_to_owners = []
            expections = []
            if self.parent.attached_df is None:
                raise Exception("No File is Attached")
            df = self.parent.attached_df[self.parent.selected_cols]
            df.columns = self.parent.cols
            verify_file(df)
            self.parent.save_api_keys(self.api_key_entry.get(), self.parent.wmt_api_key)
            from_owner = self.from_owner_entry.get()  # .split(" ")[0]
            cloud_dest = self.cloud_dest_entry.get()
            if from_owner == "" or self.parent.api_key == "":
                raise Exception("Some Fields are Empty!")
            if not self.parent.yes_no_question("This action is irreversible, Please make sure the target cloud is correct. \nAre you sure you want to continue?"):
                return
            for to_owner in df['To Owner'].unique():
                org_to_owner = to_owner
                try:
                    request_df = df[df['To Owner'] == to_owner]
                    tags_df = create_tags_df(request_df)
                    df = request_df.copy()
                    df['To Owner'] = to_owner
                    df['Cloud Destination'] = cloud_dest
                    if self.parent.tadbik == 'True':
                        to_owner = '852213717688'
                        cloud_dest = None
                    request_id = self.parent.send_request_file(from_owner, to_owner, tags_df, cloud_dest)
                    df.to_csv('data/' + request_id + '.csv', index=False)
                    request_df = pd.DataFrame([[request_id, '-', '-', '-', '-', '-', '-', str(datetime.now())[:16], cloud_dest]],
                                              columns=self.parent.requests_cols)
                    self.parent.requests_df = pd.concat([request_df, self.parent.requests_df])
                except Exception as ex:
                    failed_to_owners.append(org_to_owner)
                    expections.append(str(ex))
            if len(failed_to_owners) > 0:
                for failed_to_owner, expection in zip(failed_to_owners, expections):
                    request_df = pd.DataFrame(
                        [[expection, from_owner, failed_to_owner, 'failed', '-', '-', '-', str(datetime.now())[:16], cloud_dest]],
                        columns=self.parent.requests_cols)
                    self.parent.requests_df = pd.concat([request_df, self.parent.requests_df])
                    print(f'\n - To Owner: {failed_to_owner} reason: {expection}')
                message = f"Requests Failed For These Owners:\n {failed_to_owners}"
                message += "\n\n Please find file failed_owners.csv in the same dirictory as the input file," \
                           "\nplease fix the issues and upload again."
                df[df['To Owner'].isin(failed_to_owners)].to_csv(
                    os.path.join(os.path.dirname(self.parent.filename), "failed_owners.csv"))
                CTkMessagebox(master=self, title="Error", message=message, icon="cancel", width=500, height=200)

            self.show_page_callback()
        except Exception as ex:
            self.parent.show_error_popup(str(ex))

    def strip_columns(self, df):
        def strip(x):
            for c in [' ', '!', '@', '#', '$', '%', '^', '&', '*']:
                x = x.strip(c)
            return x

        for col in df.columns:
            df[col] = df[col].astype(str).apply(strip)

    def browse_file(self):
        try:
            filename = filedialog.askopenfilename(initialdir=__file__, title="Select Csv File",
                                                  filetypes=[("csv files", ".csv")])  #
            self.parent.attached_df = pd.read_csv(filename, dtype=str)
            self.strip_columns(self.parent.attached_df)
            self.parent.attached_df['-'] = ''
            self.parent.filename = filename
            self.display_attached_file()
        except Exception as ex:
            self.file_attach_label.configure(text=f"Error Attaching The File", text_color="red")
            raise Exception(ex)

    def update_df(self):
        self.parent.selected_cols = [cb.get() for cb in self.cbs]
        display_df = self.parent.attached_df[self.parent.selected_cols]
        display_df.columns = self.parent.cols
        if self.tree is not None:
            self.tree.destroy()
        self.tree = self.parent.create_tree_view(self, display_df, height=20)
        self.tree.place(x=50, y=240 + self.start_y)

    def display_attached_file(self):
        if self.parent.attached_df is None:
            self.file_attach_label.configure(text=f"Please Attach a File", text_color="black")
            return

        for cb, col in zip(self.cbs, self.parent.selected_cols):
            cb.configure(values=self.parent.attached_df.columns)
            cb.set(col)
        self.select_closest_col()
        self.update_df()
        self.file_attach_label.configure(text=f"File is Attached : {os.path.basename(self.parent.filename)}",
                                         text_color="green")



try:
    from tkinter import ttk
except Exception as e:
    print(f'could not import tkinter: {e}')
from wiliot_api import ManufacturingClient
from owner_change_requests_page import *
from requests_details_page import *
from send_request_multiple_reels_page import *
from send_request_one_reel_page import *
import traceback
from pathlib import Path


FILE_PATH = Path(__file__).parent.resolve()

class OwnerChangeApp(ctk.CTk):
    def __init__(self, env='prod'):
        super().__init__()
        self.env = env
        self.prev_api_key = ""
        self.api_key = ""
        self.prev_wmt_api_key = ""
        self.wmt_api_key = ""
        self.tadbik = ""
        if env in ('prod', 'test'):
            self.seperator = 'T'
        else:
            self.seperator = '-'
        self.client = None
        self.wmt_client = None
        style = ttk.Style(self)
        style.configure("Treeview.Heading", background="#39a987", foreground="black")
        screen_width = self.winfo_screenwidth()
        screen_height = self.winfo_screenheight()
        print(screen_width, screen_height)
        if screen_width > 1200:
            self.W = 1200
        else:
            self.W = screen_width - 100
        if screen_height > 800:
            self.H = 800
        else:
            self.H = screen_height - 100
        self.title("Owner Change")
        self.geometry(f"{self.W}x{self.H}")
        self.resizable(0, 0)
        ctk.set_appearance_mode("light")
        ctk.set_default_color_theme(FILE_PATH / "resources" / "wiliot-green")
        if not os.path.exists("data"):
            os.makedirs("data")
        self.setup_config()
        self.attached_df = None
        self.requests_df = None
        self.col_width_dict = col_width()
        self.selected_cols = ['-', '-', '-', '-', '-']
        self.cols = ['Ex. ID Prefix', 'First Tag', 'Last Tag', 'To Owner']
        self.requests_cols = ['Tracking ID', 'From Owner', 'To Owner', 'Status', 'Total Tags', 'Passed', 'Failed',
                              'Sent At', 'Cloud Destination']
        OwnerChangeRequests(self, self.show_send_request_multiple_reels_page).pack(fill="both", expand=True)

    def save_api_keys(self, api_key, wmt_api_key):
        config = configparser.ConfigParser()
        self.api_key = api_key
        self.wmt_api_key = wmt_api_key
        config['API'] = {'key': self.api_key,
                         'wmt_key': self.wmt_api_key,
                         'tadbik': self.tadbik}
        with open(FILE_PATH / 'resources'/ 'config.ini', 'w') as configfile:
            config.write(configfile)

    def setup_config(self):
        config = configparser.ConfigParser()
        if not os.path.exists(FILE_PATH / 'resources' / 'config.ini'):
            config['API'] = {'key': '', 'wmt_key': '', 'tadbik': 'False'}
            with open(FILE_PATH / 'resources' / 'config.ini', 'w') as configfile:
                config.write(configfile)
        config.read(FILE_PATH / 'resources'/ 'config.ini')
        self.api_key = config['API'].get('key', '')
        self.wmt_api_key = config['API'].get('wmt_key', '')
        self.tadbik = config['API'].get('tadbik', 'False')
        print("Tadbik Version:",self.tadbik)

    def get_col_width(self, col, cols, width):
        return int(self.col_width_dict[col] * width / sum([self.col_width_dict[col] for col in cols]))

    def show_error_popup(self, message, width=400, height=200):
        CTkMessagebox(master=self, title="Error", message=message, icon="cancel", width=width, height=height)
        traceback.print_exc()

    def yes_no_question(self, message, width=400, height=200):
        msgbox = CTkMessagebox(
            master=self,
            title="Confirmation",
            message=message,
            icon="question",
            width=width,
            height=height,
            option_1="Yes",
            option_2="No"
        )
        response = msgbox.get()
        return response == "Yes"

    def show_request_details_page(self, request_id, prev_frame):
        prev_frame.destroy()
        RequestDetails(self, self.show_owner_change_requests_page, request_id=request_id).pack(fill="both", expand=True)

    def show_owner_change_requests_page(self, prev_frame):
        prev_frame.destroy()
        OwnerChangeRequests(self, self.show_send_request_multiple_reels_page).pack(fill="both", expand=True)

    def show_send_request_multiple_reels_page(self, prev_frame):
        prev_frame.destroy()
        MultiReelRequestPage(self, self.show_owner_change_requests_page).pack(fill="both", expand=True)

    def show_send_request_one_reel_page(self, prev_frame):
        prev_frame.destroy()
        OneReelRequestPage(self, self.show_owner_change_requests_page).pack(fill="both", expand=True)

    def save_data(self):
        self.requests_df.to_csv(FILE_PATH / "data" / "requests.csv")

    def create_tree_view(self, frame, df, equal_width=True, height=23, width=100):
        height = int((height/23) * (self.H / 800)**2 * 23)
        tree = ttk.Treeview(frame, columns=list(df.columns), show="headings", height=height)
        for col in df.columns:
            tree.heading(col, text=col)
            if equal_width:
                tree.column(col, width=(self.W - width) // len(df.columns), anchor="center")
            else:
                tree.column(col, width=self.get_col_width(col, df.columns, self.W - width), anchor="center")

        for index, row in df.iterrows():
            tree.insert("", "end", values=list(row))
        return tree

    def check_client(self):
        if self.prev_api_key != self.api_key:
            self.client = ManufacturingClient(api_key=self.api_key, env=self.env)
            self.prev_api_key = self.api_key

        if self.prev_wmt_api_key != self.wmt_api_key and self.wmt_api_key != "":
            self.wmt_client = ManufacturingClient(api_key=self.wmt_api_key, env='wmt-prod', cloud='gcp',
                                                  region='us-central1')
            self.prev_wmt_api_key = self.wmt_api_key

    def send_request_file(self, from_owner, to_owner, df, cloud_destination=None):
        self.check_client()
        file_path = FILE_PATH / "data" / "tags_tmp.csv"
        df.to_csv(file_path)
        if cloud_destination != 'WMT':
            cloud_destination = None
        request_id = self.client.change_pixel_owner_by_file(from_owner, to_owner, file_path,
                                                            destination_cloud=cloud_destination)
        return request_id

    def get_request_aws_status(self, request_id):
        self.check_client()
        json_data = self.client.get_pixel_change_request_status(request_id)
        return json_data, json_data['status_code']

    def get_request_wmt_status(self, request_id, owner_id):
        self.check_client()
        json_data = self.wmt_client.get_import_pixel_request(owner_id, request_id)
        # print(json_data)
        return json_data, json_data['status_code']

    def send_import_request_wmt(self, request_id, owner_id):
        self.check_client()
        json_data = self.wmt_client.import_pixels(owner_id, request_id)
        return json_data

    def get_request_tags_info(self, request_id):
        file_path = FILE_PATH / "data" / "tags_tmp.csv"
        with open(file_path, "w") as f:
            self.check_client()
            self.client.get_pixel_change_request_details(request_id, f)
        df = pd.read_csv(file_path)
        return df

if __name__ == "__main__":
    app = OwnerChangeApp('prod')
    app.mainloop()


from wiliot_api import ManufacturingClient

from wiliot_tools.tadbik_label_printers.assembly.data.default_values import empty_page, default_config
from wiliot_tools.utils.wiliot_gui.wiliot_gui import WiliotGui
from wiliot_tools.tadbik_label_printers.label_printer_tools import *


class AssemblyLabelPrinter:
    def __init__(self, env="prod"):
        self.sensor = YoctoSensor()
        self.sensor.connect()
        self.counter = 0
        self.last_assembly_id = None
        self.history = self.read_history()
        self.config = self.read_config()
        self.client = get_manufacturing_client(env)
        self.init_widgets()
        self.app.layout.title("assembly Label Printer")
        self.events()
        self.app.add_recurrent_function(cycle_ms=1000, function=self.recurrent)
        self.load_assembly_data()
        self.app.set_parent(self)
        self.inputs_security()
        self.app.run()

    def init_widgets(self):
        main_dict = {}
        main_dict["assembly_id"] = list(self.history.keys())[::-1]
        self.last_assembly_id = main_dict["assembly_id"][0]
        main_dict["operator_name"] = ['-'] + self.config['operator_name']
        main_dict["inlay"] = ['-'] + self.config['inlay']
        main_dict["assembled_reel_width"] = ''
        main_dict["#_of_lanes_in_antenna_design"] = ['-'] + self.config['#_of_lanes_in_antenna_design']
        main_dict["assembled_reel_count"] = ['-'] + self.config['assembled_reel_count']
        main_dict.update({k: {'value': v} for k, v in main_dict.items()})
        main_dict["warning"] = {'value': '-', 'widget_type': 'label', 'options': {'fg': 'red'}, 'columnspan': 2}
        main_dict["assembled_reel_width"] = {'value': '', 'text': 'Assembled reel width (e.g. 1,1,2)'}
        main_dict['tabs'] = {}

        types = ['Placement', 'Removal']
        wafers = ['First', 'Second', 'Third']
        tabs_dict = {}
        for i, wafer in enumerate(wafers):
            for j, type in enumerate(types):
                tab_dict = {}
                tab_name = wafer + ' ' + type
                tab_dict["wafer_qr_code"] = ''

                if type == 'Removal':
                    tab_dict['wafer_dies_qty_assembled'] = ''
                tab_dict['record_time'] = ''
                tab_dict["temperature"] = ''
                tab_dict["humidity"] = ''
                tab_dict["date"] = ''
                x = str(i * 2 + j + 1) + ') '
                tabs_dict.update({x + k: {'value': v, 'tab': tab_name} for k, v in tab_dict.items()})

                tabs_dict[x + 'record_time'] = {'value': 'Record Date & Sensors', 'widget_type': 'button',
                                                'columnspan': 2,
                                                'tab': tab_name}
                tabs_dict[x + 'next'] = {'value': 'Next', 'widget_type': 'button', 'columnspan': 1,
                                         'tab': tab_name}
        main_dict['total_dies_qty_assembled'] = {'value': 0, 'disabled': True}
        main_dict['assembly_label'] = {'value': '', 'disabled': True}
        main_dict['wafers'] = {'value': '', 'disabled': True}
        main_dict['print_label'] = {'value': title_button('Print Label', 25), 'widget_type': 'button',
                                    'columnspan': 2}
        main_dict['save_and_start_new_assembly'] = {'value': 'Save & Start New assembly', 'widget_type': 'button',
                                                    'columnspan': 2}
        WiliotGui.format_label = lambda self, x: x.replace('_', ' ').capitalize() if ')' not in x else x.split(")")[1][
                                                                                                       1:].replace('_',
                                                                                                                   ' ').capitalize()
        main_dict['tabs'] = tabs_dict
        main_dict["output"] = {'value': '', 'widget_type': 'label', 'columnspan': 2}

        self.app = WiliotGuiCustomized(main_dict, full_screen=False, theme='wiliot', do_button_config=False)

    def events(self):
        for i in range(1, 7):
            self.app.add_event(
                widget_key=f'tabs_{i}) record_time',
                event_type='button',
                command=lambda i=i: self.update_date_and_sensors(i))
            self.app.add_event(
                widget_key=f'tabs_{i}) next',
                event_type='button',
                command=self.go_to_next_tab
            )
        self.app.add_event(widget_key=f'print_label', event_type='button', command=self.print_label)
        self.app.add_event(widget_key=f'save_and_start_new_assembly', event_type='button',
                           command=self.save_and_start_new_assembly)
        self.app.add_event(widget_key=f'assembly_id', event_type='<<ComboboxSelected>>', command=self.save_and_load)

    def update_date_and_sensors(self, i):
        self.app.update_widget(widget_key=f'tabs_{i}) date', new_value=get_israel_time())
        temp = self.sensor.get_temperature()
        if temp is None:
            temp = 'Not Connected'
        hum = self.sensor.get_humidity()
        if hum is None:
            hum = 'Not Connected'
        self.app.update_widget(widget_key=f'tabs_{i}) temperature', new_value=temp)
        self.app.update_widget(widget_key=f'tabs_{i}) humidity', new_value=hum)

    def go_to_next_tab(self):
        notebook = self.app.widgets_tabs['tabs']['layout']
        current_tab = notebook.index(notebook.select())
        total_tabs = notebook.index('end')

        next_tab = (current_tab + 1) % total_tabs
        notebook.select(next_tab)

    def inputs_security(self):
        dropdowns = ['assembly_id', 'operator_name', 'inlay', '#_of_lanes_in_antenna_design', 'assembled_reel_count']
        for dd in dropdowns:
            self.app.widgets[dd].config(state='readonly')

        def only_numeric_input(input_value):
            if input_value == "" or all(char.isdigit() for i, char in enumerate(input_value)):
                return True
            return False

        vcmd = (self.app.class_tk.register(only_numeric_input), '%P')
        for i in [2, 4, 6]:
            self.app.widgets[f'tabs_{i}) wafer_dies_qty_assembled'].config(validate='key', validatecommand=vcmd)

        def only_numeric_input(input_value):
            if input_value == "" or all(
                    (char.isdigit() and i % 2 == 0) or (char in (',', ',') and i % 2 == 1) for i, char in
                    enumerate(input_value)):
                return True
            return False

        vcmd = (self.app.class_tk.register(only_numeric_input), '%P')
        self.app.widgets[f'assembled_reel_width'].config(validate='key', validatecommand=vcmd)

    def check_width_lanes(self, values):
        width = values['assembled_reel_width']
        lanes = values['#_of_lanes_in_antenna_design']
        assembled_reel_count = values['assembled_reel_count']
        width = width.replace(",", ",")
        if width == '' or lanes == '-':
            self.app.update_widget(widget_key='warning', new_value='')
            return
        if width[-1] == ',':
            width = width[:-1]
        widths = [int(x) for x in width.split(',')]
        width = sum(widths)
        lanes = int(lanes)
        if width != lanes:
            self.app.update_widget(widget_key='warning',
                                   new_value=f'The sum of assembled reel widths ({width}) does not match the number of lanes ({lanes}).')
        elif assembled_reel_count != '-' and int(assembled_reel_count) != len(widths):
            self.app.update_widget(widget_key='warning',
                                   new_value=f'The number of assembled reel widths ({len(widths)}) does not match the assembled reel count ({assembled_reel_count}).')
        else:
            self.app.update_widget(widget_key='warning', new_value='')

    def sum_counts(self, values):
        sum_vals = 0
        for i in [2, 4, 6]:
            val = values.get(f'tabs_{i}) wafer_dies_qty_assembled', 0)
            try:
                val = int(val)
            except ValueError:
                val = 0
            sum_vals += val
        self.app.update_widget(widget_key=f'total_dies_qty_assembled', disabled=False)
        self.app.update_widget(widget_key=f'total_dies_qty_assembled', new_value=sum_vals, disabled=True)

    def update_assembly_label(self, values):
        assembly_id = values['assembly_id']

        inlay = values['inlay']
        if inlay == '-':
            inlay = '???'

        lanes_in_antenna_design = values['#_of_lanes_in_antenna_design']
        if lanes_in_antenna_design == '-':
            lanes_in_antenna_design = '?'

        dates = [values[f'tabs_{i}) date'] for i in range(1, 7) if values[f'tabs_{i}) date'] != '']
        if len(dates) == 0:
            end_date = None
        else:
            end_date = format_date(max(dates))
        if end_date is None:
            end_date = "????????"

        assembled_reel_count = values['assembled_reel_count']
        if assembled_reel_count == '-':
            assembled_reel_count = '?'

        assembly_label = f'{inlay}_{lanes_in_antenna_design}_{end_date}_{assembled_reel_count}'
        if '?' in assembly_label:
            run_num = '?'
        else:
            labels = set([y['assembly_label'] for x, y in self.history.items() if x != assembly_id])
            run_num = 1
            while f"{assembly_label}_{run_num}" in labels:
                run_num += 1
        assembly_label = f"{assembly_label}_{run_num}"
        self.app.update_widget(widget_key=f'assembly_label', disabled=False)
        self.app.update_widget(widget_key=f'assembly_label', new_value=assembly_label, disabled=True)

    def update_wafers(self, values):
        wafers = self.get_wafers(values)
        wafers = ", ".join(wafers)
        self.app.update_widget(widget_key=f'wafers', disabled=False)
        self.app.update_widget(widget_key=f'wafers', new_value=wafers, disabled=True)

    def recurrent(self):
        values_out = self.app.get_all_values()
        self.check_width_lanes(values_out)
        self.sum_counts(values_out)
        self.update_assembly_label(values_out)
        self.update_wafers(values_out)
        if self.counter % 10 == 9:
            self.save_history(output_message=False)

        self.counter += 1

    @staticmethod
    def read_history():
        try:
            with open(os.path.join('data', 'assembly_history.json'), 'r') as file:
                file = json.load(file)
            return file
        except FileNotFoundError:
            return {"1": {}}

    @staticmethod
    def read_config():
        try:
            with open('config.json', 'r') as file:
                file = json.load(file)
            return file
        except FileNotFoundError:
            with open('config.json', 'w') as file:
                json.dump(default_config, file, indent=4)
            return default_config

    def save_page(self, output_message=True):
        values = self.app.get_all_values()
        assembly_id = str(values.get("assembly_id"))
        self.history[assembly_id] = values
        if output_message:
            print("Saved current page successfully.")

    def save_history(self, output_message=True):
        try:
            self.save_page(output_message)
            with open(os.path.join('data', 'assembly_history.json'), 'w') as file:
                json.dump(self.history, file, indent=4)
            if output_message:
                print("History saved successfully.")
        except Exception as e:
            print(f"An error occurred while saving history: {e}")

    def load_assembly_data(self):
        assembly_id = self.app.get_all_values().get("assembly_id")
        data = self.history.get(assembly_id, {})
        for key, value in data.items():
            self.app.update_widget(widget_key=key, new_value=value)

    def reset_page(self):
        for key, value in empty_page.items():
            self.app.update_widget(widget_key=key, new_value=value)

    def save_and_start_new_assembly(self):
        self.save_history()
        last_id = list(self.history.keys())[::-1][0]
        self.reset_page()
        assembly_id = str(int(last_id) + 1)
        self.app.update_widget(widget_key=f'assembly_id', new_value=assembly_id,
                               options=[assembly_id] + list(self.history.keys())[::-1])
        self.last_assembly_id = assembly_id

    def save_and_load(self, _):
        values = self.app.get_all_values()
        assembly_id = values.get("assembly_id")
        values["assembly_id"] = self.last_assembly_id
        self.history[self.last_assembly_id] = values
        self.last_assembly_id = assembly_id
        self.load_assembly_data()



    def get_wafers(self, values):
        try:
            wafers = []
            for i in [1, 3, 5]:
                wafer = values[f'tabs_{i}) wafer_qr_code']
                if wafer == '':
                    continue
                lot = wafer.split('.')[0]
                wafer = wafer.split('_')[-1]
                if len(lot) != 9 or not int(wafer):
                    raise ValueError(f"Invalid wafer {wafer}")
                wafer = f"{lot}.{wafer}"
                wafers.append(wafer)
            return wafers
        except Exception as e:
            return ['Invalid QR Code']

    def verify_values(self, values):
        error_messages = []

        if values.get('operator_name') == '-':
            error_messages.append('Please select an operator name.')

        if values.get('inlay') == '-':
            error_messages.append('Please select an inlay.')

        if values.get('#_of_lanes_in_antenna_design') == '-':
            error_messages.append('Please select the number of lanes in antenna design.')

        if values.get('assembled_reel_count') == '-':
            error_messages.append('Please select an assembled reel count.')

        widths_str = values.get('assembled_reel_width', '').replace(',', ',')
        if not widths_str:
            error_messages.append('Please enter the assembled reel width.')
        else:
            widths_list = widths_str.strip(',').split(',')
            try:
                widths = [int(w.strip()) for w in widths_list]
            except ValueError:
                error_messages.append('Assembled reel width must be a comma-separated list of integers.')
            else:
                lanes_str = values.get('#_of_lanes_in_antenna_design')
                try:
                    lanes = int(lanes_str)
                except ValueError:
                    error_messages.append('Invalid number of lanes in antenna design.')
                else:
                    if sum(widths) != lanes:
                        error_messages.append(
                            f'The sum of assembled reel widths ({sum(widths)}) does not match the number of lanes ({lanes}).')
                    else:
                        assembled_reel_count = values.get('assembled_reel_count')
                        if assembled_reel_count != '-' and int(assembled_reel_count) != len(widths):
                            error_messages.append(
                                f'The number of assembled reel widths ({len(widths)}) does not match the assembled reel count ({assembled_reel_count}).')

                    inlay = values.get('inlay')
                    inlay_lanes = self.config['inlay_rows'].get(inlay)
                    if lanes != inlay_lanes:
                        error_messages.append(
                            f'The lanes in antenna design ({lanes}) does not match the number of lanes for the inlay {inlay} ({inlay_lanes}).')

        dates = []
        for i in range(1, 7):
            key_qr_code = f'tabs_{i}) wafer_qr_code'
            key_qty = f'tabs_{i}) wafer_dies_qty_assembled'
            key_date = f'tabs_{i}) date'
            key_temperature = f'tabs_{i}) temperature'
            key_humidity = f'tabs_{i}) humidity'

            if not values.get(key_qr_code) and i % 2 == 1 and i != 1:
                break

            if not values.get(key_qr_code):
                error_messages.append(f'Please enter wafer QR code in tab {i}.')

            if i % 2 == 0:
                if values.get(f'tabs_{i-1}) wafer_qr_code') != values.get(key_qr_code):
                    error_messages.append(f"QR codes in tab {i-1} and {i} don't match")
                qty_str = values.get(key_qty)
                if not qty_str:
                    error_messages.append(f'Please enter wafer dies quantity assembled in tab {i}.')
                else:
                    try:
                        qty = int(qty_str)
                        if qty <= 0:
                            raise ValueError
                    except ValueError:
                        error_messages.append(
                            f'Invalid wafer dies quantity assembled in tab {i}. Please enter a positive integer.')

            if not values.get(key_date):
                error_messages.append(f'Please record date and sensors in tab {i}.')
            else:
                dates.append(values.get(key_date))

            if not values.get(key_date) or not values.get(key_temperature) or not values.get(key_humidity):
                error_messages.append(f'Please record date and sensors in tab {i}.')

        wafers = self.get_wafers(values)
        if 'Invalid QR Code' in wafers:
            error_messages.append('Invalid wafer QR code(s). Please check the wafer QR codes.')

        if not wafers or wafers == ['']:
            error_messages.append('Please enter at least one valid wafer QR code.')

        if len(wafers) != len(set(wafers)):
            error_messages.append('Please make sure that wafer names are different.')

        if str(dates) != str(sorted(dates)):
            error_messages.append('Please make sure that the dates are ascending.')

        if error_messages:

            full_error_message = error_messages[0]
            self.app.update_widget(widget_key='output', new_value=full_error_message, color='red')
            return False
        else:
            return True

    def print_label(self):
        values = self.app.get_all_values()
        self.update_assembly_label(values)
        self.update_wafers(values)
        values = self.app.get_all_values()
        if not self.verify_values(values):
            return
        assembly_label = values['assembly_label']
        wafers = values['wafers'].split(", ")
        wafers = wafers + [''] * (3 - len(wafers))
        total_dies_qty_assembled = int(values.get("total_dies_qty_assembled"))
        assembled_reel_count = int(values.get("assembled_reel_count"))
        lanes_in_antenna_design = int(values.get("#_of_lanes_in_antenna_design"))
        qty_per_reel = total_dies_qty_assembled // lanes_in_antenna_design
        qr_code = f"{assembly_label}_{'_'.join(wafers)}"
        label = f"""
                     ^XA
                    ^FO0,0^GFA,3024,3024,27,,::::::::::::::::::::::gK01FCQ03F8,::::gK01FCP0F3F8,gK01FCO03IF8,gK01FCO07IF8,:gK01FCO0F0FF8,gK01FCO0F07F8,gK01FCO0E07F8,:gK01FCO0F07F8,gK01FCO0F8FF8,gK01FCO07IF8,O0FEI0FE001FC07F01FC07FL03IF8,O0FEI0FE001FC07F01FC07FL01JFC,O0FEI0FE001FC07F01FC07FJ0EI03FFC,O0FEI0FE001FC07F01FC07FI0FFE003FFC,O0FEI0FE001FC07F01FC07F003IF003FFC,O0FEI0FE001FC07F01FC07F007IFC03FFC,O0FEI0FE001FC07F01FC07F00JFE03FFC,O0FEI0FE001FCJ01FCJ01KF03FFC,O0FEI0FE001FCJ01FCJ03KF03F8,O0FEI0FE001FCJ01FCJ03KF83F8,O0FEI0FE001FC07F01FC07F07FC07F83F8,O0FEI0FE001FC07F01FC07F07F803FC3F8,O0FEI0FE001FC07F01FC07F0FF001FC3F8,O0FEI0FE001FC07F01FC07F0FE001FC3F8,O0FEI0FE001FC07F01FC07F0FE001FE3F8,::O0FEI0FE001FC07F01FC07F0FE001FC3F8,O0FEI0FE001FC07F01FC07F0FF001FC3F8,O0FEI0FE001FC07F01FC07F07F003FC3F8,O0FEI0FE001FC07F01FC07F07F807FC3F8,O0FEI0FE001FC07F01FC07F07FE1FF83F8,O0FEI0FE001FC07F01FC07F03KF83F8,O0FEI0FE001FC07F01FC07F01KF03F8,O0FEI0FE001FC07F01FC07F01JFE03F8,O0FEI0FE001FC07F01FC07F00JFC03F8,O0FEI0FE001FC07F01FC07F003IF803F8,O0FEI0FE001FC07F01FC07F001FFE003F8,O0FE001FE001FC07F01FC07FI03F8003F8,O0FF001FE001FC07F01FC07FN03F8,O07F003FF003FC07F01FC07FN03FC,O07F803FF807F807F01FC07FN01FE,O07FE0IFC0FF807F01FC07FN01FF03,O03PF007F01FC07FO0JF8,O01PF007F01FC07FO0JFC,O01OFE007F01FC07FO07IFE,P0JFEJFC007F01FC07FO03JF,P07IF87IF8007F01FC07FO01IFE,P01IF03FFEI07F01FC07FP07FFC,Q07FC007F8I07F01FC07FP01FE,,::::::::::::::::::::::::::::::::^FS
                    ^LH0,0
                            ^FO420,15^BQN,2,5,L
                    ^FDMA,{qr_code}^FS
                    ^FO370,200^A0N,30,30^FD{assembly_label}^FS
                    ^FO30,200^A0N,40,40^FDBEFORE CONV^FS
                    ^FO30,90^A0N,28,28^FD{wafers[0]}^FS
                    ^FO30,125^A0N,28,28^FD{wafers[1]}^FS
                    ^FO30,160^A0N,28,28^FD{wafers[2]}^FS
                    ^FO230,90^A0N,28,28^FDQty Per Lane:^FS
                    ^FO230,125^A0N,28,28^FD{qty_per_reel}^FS
                    ^XZ
                    """
        for i in range(assembled_reel_count):
            if print_label(label, self.config.get("printer_name")):
                self.app.update_widget(widget_key='output',
                                       new_value=f'Labels successfully sent to printer on {datetime.now()}', color='green')
            else:
                self.app.update_widget(widget_key='output', new_value='Failed to connect to the printer.', color='red')
        upload_json_to_s3(self.client, values, assembly_label, 'assembly')


if __name__ == '__main__':
    AssemblyLabelPrinter()



default_config = {
        "operator_name": ['Adrian', 'Liel', 'Yaniv', 'Yael'],
        "inlay": [117, 169],
        "printer_name": "",
        "converted_reel_name_length": 18,
        "sample_test_first_ext_id_length": 8,
        "sample_test_last_ext_id_length": 8,
        "fg_reel_first_ext_id_length": 8,
        "separator": "T",
}


from shipment_approval_main import ShipmentApprovalApp
import json
try:
    from PIL import Image, ImageTk
    import customtkinter as ctk
except Exception as e:
    print(f'could not import tkinter: {e}')
import pandas as pd
import os
import subprocess
import threading

if __name__ == "__main__":
    app = ShipmentApprovalApp()
    app.mainloop()



import time

from wiliot_core import set_logger
from wiliot_tools.test_equipment.test_equipment import YoctoSensor

TIME_BTWN_MEASURE = 1  # seconds
path, logger = set_logger(app_name='GetHumidity', dir_name='humidity_sensor', file_name='get_humidity')

sensor = YoctoSensor(logger)

while True:
    humidity = sensor.get_humidity()
    logger.info(humidity)
    time.sleep(TIME_BTWN_MEASURE)